{
  en => {
    errors => {
      attributes => {
        date_of_birth => {
          not_future => 'Date {{value}} is future',
        },
      },
      messages => {
        not_voting_age => 'not voting age',
      }
    }
  },
};
