{
  en => {
    errors => {
      messages => {
        #result_set
        too_few => 'has too few rows (minimum is {{min}})',
        too_many => 'has too many rows (maximum is {{max}})',
      },
    }
  },
};
